package volunteerdbconnector;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.ArrayList;
import javafx.application.Application;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellDataFeatures;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;
import javafx.util.Callback;

/**
 * @author Gerhard Scheiber
 */

public class VolunteerDbConnector extends Application 
{

    private ObservableList<ObservableList> data;
    private TableView tableview;
    private Pane viewVolunteer;
    ObservableList<String> dropDownOptions;
    ObservableList<String> dropDownOptionsIDs;

    public static void main(String[] args) 
    {
        launch(args);
    }

    public void buildData() 
    {
        Connection c;
        data = FXCollections.observableArrayList();
        try 
        {
            c = DBConnect.connect();

            String SQL_getAll = "select v.prename, v.surname, t.german\n"
                    + "  from volunteer v\n"
                    + "  left join volunteer_event ve on v.id = ve.volunteer_id\n"
                    + "  join c_area_of_interest ai on ve.selectedCAreaOfInterest_id = ai.id\n"
                    + "  join translation t on ai.translation_tid = t.tid";

            String SQL_DropDownValues = "select t.german\n"
                    + "  from translation t\n"
                    + "  join c_area_of_interest aoi on aoi.translation_tid = t.tid"
                    + "  order by t.german asc";

            String SQL_DropDownValuesID = "select aoi.id\n"
                    + "  from c_area_of_interest aoi\n"
                    + "  join translation t on aoi.translation_tid = t.tid"
                    + "  order by t.german asc";
            
            ResultSet rs = c.createStatement().executeQuery(SQL_getAll);
            ResultSet rs_dd = c.createStatement().executeQuery(SQL_DropDownValues);
            ResultSet rs_ddid = c.createStatement().executeQuery(SQL_DropDownValuesID);

            ArrayList<String> list = new ArrayList();
            while (rs_dd.next()) 
            {
                for (int i = 1; i <= rs_dd.getMetaData().getColumnCount(); i++) 
                {
                    list.add(rs_dd.getString(i));
                }
            }
            
            ArrayList<String> idList = new ArrayList();
            while (rs_ddid.next()) 
            {
                for (int i = 1; i <= rs_ddid.getMetaData().getColumnCount(); i++) 
                {
                    idList.add(rs_ddid.getString(i));
                }
            }

            //System.out.println("Areas of interest" + list);
            dropDownOptions = FXCollections.observableArrayList(list);
            dropDownOptionsIDs = FXCollections.observableArrayList(idList);
            
            for (int i = 0; i < rs.getMetaData().getColumnCount(); i++) 
            {
                final int j = i;
                TableColumn col = new TableColumn(rs.getMetaData().getColumnName(i + 1));
                col.setCellValueFactory(new Callback<CellDataFeatures<ObservableList, String>, ObservableValue<String>>() 
                {
                    public ObservableValue<String> call(CellDataFeatures<ObservableList, String> param) 
                    {
                        return new SimpleStringProperty(param.getValue().get(j).toString());
                    }
                });

                tableview.getColumns().addAll(col);
                System.out.println("Column [" + i + "] ");
            }

            while (rs.next()) 
            {

                ObservableList row = FXCollections.observableArrayList();
                for (int i = 1; i <= rs.getMetaData().getColumnCount(); i++) 
                {
                    row.add(rs.getString(i));
                }

                //hier sollte das DropDown-Feld mit übergeben werden - zum Testen wird der Konsole dropdownoptions übergeben
                //row.add(dropDownOptions);
                System.out.println("Row [1] added " + row);
                data.add(row);

            }

            tableview.setItems(data);
        } catch (Exception e) 
        {
            e.printStackTrace();
            System.out.println("Error on Building Data");
        }
    }

    @Override
    public void start(Stage stage) throws Exception 
    {

        tableview = new TableView();
        viewVolunteer = new Pane();
        buildData();

        Scene scene = new Scene(tableview, 800, 600);
        Scene vScene = new Scene(viewVolunteer, 400, 100);

        stage.setScene(scene);
        stage.show();

        //Checking if a row gets double clicked + output in console
        tableview.setOnMousePressed(new EventHandler<MouseEvent>() 
        {

            @Override
            public void handle(MouseEvent event) 
            {
                if (event.isPrimaryButtonDown() && event.getClickCount() == 2) 
                {
                    System.out.println(tableview.getSelectionModel().getSelectedItem());
                    String volunteer = tableview.getSelectionModel().getSelectedItem().toString();
                    Label label = new Label(volunteer);
                    viewVolunteer.getChildren().add(label);
                    ComboBox<KeyValuePair> comboBox = new ComboBox<KeyValuePair>();
                    for (int i = 0; i < dropDownOptions.size(); i++)
                    {
                        comboBox.getItems().add(new KeyValuePair(dropDownOptionsIDs.get(i), dropDownOptions.get(i)));
                    }
                    //comboBox.setItems(dropDownOptions);
                    comboBox.setLayoutX(0);
                    comboBox.setLayoutY(20);
                    Button btn = new Button("Add");
                    btn.setLayoutX(50);
                    btn.setLayoutY(50);
                    btn.setOnAction(new EventHandler<ActionEvent>() 
                    {
                        @Override
                        public void handle(ActionEvent event) 
                        {
                            System.out.println("Jetzt soll AOI# " + comboBox.getValue().getKey() + " gespeichert werden!");
                            label.setText("");
                            stage.setScene(scene);
                        }
                    });
                    viewVolunteer.getChildren().add(btn);
                    viewVolunteer.getChildren().add(comboBox);

                    stage.setScene(vScene);

                }
            }

        });

    }
}
